
#include "stdafx.h"
#include "CheckBox.h"
#include <crtdbg.h>

#define _MY_ASSERT(value) if(!value) { _ASSERTE(value != 0); return; }


//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// Initialisation of static members

bool		CCheckBox::drawFocus = true;
HINSTANCE	CCheckBox::m_hInst   = NULL;



//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// public methods

//-----------------------------------------------------------------------------
// Name:		CCheckBox
// Description:	Constructor
//
// Entry:		
//				int rcID		- resource ID with state bitmaps
//
//				int dimension	== width == height of one state bitmap
//-----------------------------------------------------------------------------
CCheckBox::CCheckBox (int rcID,int dimension) : 
  m_rcID( rcID ), 
  BOX_SIZE( dimension )
{	
	_ASSERTE (BOX_SIZE > 0);
	_ASSERTE (BOX_SIZE < 30);

	m_bLeftText = false;
	m_bDisabled = false;
	m_bChecked  = false;
	m_nState    = BOX_OFF;

	inRect    = false;
	spaceDown = false;
	mouseDown = false;

	m_hBox	= NULL;
	m_hDlg	= NULL;
	m_brush = NULL;

	oProc = NULL;	
	hasFocus = false;

	prepare_bitmaps = true;

	for (int i=0; i<6; i++)
		m_bitmaps [i] = 0;
}


//-----------------------------------------------------------------------------
// Name:		Subclass
// Description:	Calls "Subclass_X" where the actual subclassing and some 
//				other initialisations are done.
//				Additionaly "Subclass_X" checks the following:
//					1. CCheckBox::m_hInst  initialised?
//					2. bitmap with CB states successfully loaded?
//				If any of this fails the CB is not subclassed and an 
//				assertion is generated if _DEBUG is defined (debug version).
//
// Entry:		
//				UINT nID  - resource ID of CB (to subclass)
//
//				HWND hCB  - handle to CB (to subclass)
//
//				HWND hDlg - parent of (subclassed) CB
//-----------------------------------------------------------------------------
void CCheckBox::Subclass (UINT nID, HWND hDlg)
{	
	m_hBox = ::GetDlgItem (hDlg,nID);
	m_hDlg = hDlg;	

	Subclass_X ();
}

void CCheckBox::Subclass (HWND hCB, HWND hDlg)
{	
	m_hBox = hCB;
	m_hDlg = hDlg;	

	Subclass_X ();
}


//-----------------------------------------------------------------------------
// Name:		~CCheckBox
// Description:	Destructor
//-----------------------------------------------------------------------------
CCheckBox::~CCheckBox()
{
	SetWindowLong (m_hBox,GWL_WNDPROC, (long)oProc);
	SetWindowLong (m_hBox,GWL_USERDATA,0);

	for (int i=0; i<6; i++) {   
		if (m_bitmaps [i])
			DeleteObject (m_bitmaps [0]);
	}	

	DeleteObject (m_font);
}


//-----------------------------------------------------------------------------
// Name:		SetCheck
// Description:	Sets or resets the check state
//
// Entry:		
//				int nCheck - this parameter can be one of the following:
//					 1 :   checked state
//					 0 :   unchecked state
//-----------------------------------------------------------------------------
void CCheckBox::SetCheck (int nCheck)
{
	m_bChecked = (nCheck == 1) ? true : false;
		
	if (m_bChecked) {

		if (m_bDisabled) {              // the state of a disabled CB
			SetState (BOX_DISABLED_1);  // can also be changed
			return;
		}

		if (mouseDown)					// if mouse down => space doesn't matter
		{
			if (inRect)	SetState (BOX_LDOWN_1);
			else  		SetState (BOX_ON);
		}
		else if (spaceDown)
			 SetState (BOX_LDOWN_1);
		else SetState (BOX_ON);
	}
	else {
		if (m_bDisabled) {
			SetState (BOX_DISABLED_2);
			return;
		}

		if (mouseDown)
		{
			if (inRect)	SetState (BOX_LDOWN_2);
			else  		SetState (BOX_OFF);
		}
		else if (spaceDown)
			 SetState (BOX_LDOWN_2);
		else SetState (BOX_OFF);
	}
}



//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//
// protected & private methods


//-----------------------------------------------------------------------------
// Name:		Subclass_X
// Description:	The actual subclassing and some other initialisations
//				are done in this method
//				Additionaly the following is checked:
//					1. CCheckBox::m_hInst  initialised?
//					2. bitmap with CB states successfully loaded?
//				If any of this fails the CB is not subclassed and an 
//				assertion is generated if _DEBUG is defined (debug version).
//-----------------------------------------------------------------------------
void CCheckBox::Subclass_X ()
{
	_MY_ASSERT (m_hInst);	// * m_hInst initialised ?		*

	m_bSrc = LoadBitmap (m_hInst,MAKEINTRESOURCE(m_rcID));	// load state bitmaps	
	_MY_ASSERT (m_bSrc);	// * bitmap resource ID valid ? *


	// text on left side?
	//
	long style    = ::GetWindowLong (m_hBox,GWL_STYLE);
	long ex_style = ::GetWindowLong (m_hBox,GWL_EXSTYLE);

	if ((style	  & BS_LEFTTEXT)    || 
		(style	  & BS_RIGHTBUTTON) || 
		(ex_style & WS_EX_RIGHT) )
		m_bLeftText = true;
	

	GetWindowText (m_hBox, m_text, 64);							// get text
	m_font = (HFONT) SendMessage (m_hDlg, WM_GETFONT, 0, 0);	// get font

	if (!IsWindowEnabled (m_hBox)) {
		m_bDisabled = true;
		m_nState = BOX_DISABLED_2;
	}
	
	// prevent any drawing by the control itself
	//
	SetWindowLong (m_hBox,GWL_STYLE,
				   GetWindowLong (m_hBox,GWL_STYLE) | BS_OWNERDRAW);
	

	// set new window procedure for the check box (subclass it)
	//
	oProc = (WNDPROC) ::SetWindowLong (m_hBox,GWL_WNDPROC, (long)StaticProc);


	// give m_hBox a this pointer, so it can later determine
	// to which instance of CCheckBox it belongs
	//
	SetWindowLong (m_hBox,GWL_USERDATA, (long)this);
}


//-----------------------------------------------------------------------------
// Name:		StaticProc
// Description:	A static method of type WNDPROC. It's only purpose is to call
//				the WndProc of the corresponding CCheckBox instance.
//-----------------------------------------------------------------------------
LRESULT CALLBACK CCheckBox::StaticProc (HWND hWndBox, UINT uMsg,WPARAM wParam,LPARAM lParam)
{	
	CCheckBox* box = (CCheckBox*) ::GetWindowLong (hWndBox,GWL_USERDATA);
	return box ->WndProc (hWndBox,uMsg,wParam,lParam);
}


//-----------------------------------------------------------------------------
// Name:		WndProc
// Description:	New WndProc of the subclassed CB
//-----------------------------------------------------------------------------
LRESULT CCheckBox::WndProc (HWND hWndBox, UINT uMsg,WPARAM wParam,LPARAM lParam)
{	
	RECT  rc;
	POINT pt;	
			

	if (uMsg == WM_MOUSEMOVE || uMsg == WM_LBUTTONUP)
	{
		pt.x = LOWORD(lParam);
		pt.y = HIWORD(lParam);
		::GetClientRect (hWndBox, &rc);
	}


	switch (uMsg) {

	case BM_GETCHECK:
		if (m_bChecked) return BST_CHECKED;
		else			return BST_UNCHECKED;

	case BM_SETCHECK:			
		if (wParam == BST_CHECKED)
			SetCheck (1);
		else 
			SetCheck (0);		
		return 0;

	case WM_PAINT:
		{
			DWORD clr1 = ::GetSysColor (COLOR_3DFACE);
			if (m_clr != clr1) {			// sysColor changed ??				
				for (int i=0; i<6; i++) {	// => prepare bitmaps again
					if (m_bitmaps [i])
						DeleteObject (m_bitmaps [0]);
				}
				m_clr = clr1;

				if (m_brush) DeleteObject (m_brush);
				m_brush = ::CreateSolidBrush (m_clr);

				prepare_bitmaps = true;
			}


			if (hasFocus)
				DrawFocus (true);

			PAINTSTRUCT ps;
			HDC hdc = ::BeginPaint (hWndBox, &ps);
			if (prepare_bitmaps) {			// called first time or sysColor changed?
				PrepareBitmaps (hdc);		// => prepare bitmaps
				prepare_bitmaps = false;
			}
			BlitToScreen (hdc,m_nState);
			::EndPaint (hWndBox, &ps);

			if (hasFocus)
				DrawFocus (true);
		}
		return 0;

	case WM_ERASEBKGND:
		return true;
		
	case WM_ENABLE:		
		if (wParam) {
			m_bDisabled = false;

			if (m_nState == BOX_DISABLED_1)
				 SetState (BOX_ON);
			else SetState (BOX_OFF);			
		}
		else {
			m_bDisabled = true; 

			if (m_bChecked)
				 SetState (BOX_DISABLED_1);
			else SetState (BOX_DISABLED_2);
		}
		break;

	case WM_KEYDOWN:
		if (wParam == VK_SPACE && !spaceDown) {

			spaceDown = true;

			if (!mouseDown)
			{		
				if (m_bChecked) SetState (BOX_LDOWN_1);
				else			SetState (BOX_LDOWN_2);
			}
		}
		else if (wParam != VK_SPACE && spaceDown) { // if any other key pressed
													// while space down
            spaceDown = false;			

			if (!mouseDown)
			{		
				if (m_bChecked) SetState (BOX_ON);
				else			SetState (BOX_OFF);
			}			
		}
		break;

	case WM_KEYUP:
		if (wParam == VK_SPACE && spaceDown) {

			spaceDown = false;

			if (!mouseDown) {
				m_bChecked = (m_bChecked == true ? false : true);

				if (m_bChecked) SetState (BOX_ON);
				else			SetState (BOX_OFF);				
			}			
		}
		break;


	case WM_LBUTTONDBLCLK:		// BS_OWNERDRAW buttons send BN_DBLCLK notifications;						
		uMsg = WM_LBUTTONDOWN;	// to prevent this, we pass WM_LBUTTONDOWN to the
								// default WNDPROC (oProc)
	case WM_LBUTTONDOWN:

		mouseDown = true;
		inRect = true;		

		if (m_bChecked) SetState (BOX_LDOWN_1);
		else			SetState (BOX_LDOWN_2);
		
					// now call the default WNDPROC => 
		break;      // it calls "SetCapture" & "SetFocus"
	

	case WM_MOUSEMOVE:
		if (mouseDown)
		{
			if (!PtInRect (&rc, pt)) {

				if (m_bChecked) SetState (BOX_ON);
				else			SetState (BOX_OFF);
				
				inRect = false;
			}
			else if (!inRect)  // again over control
			{
				inRect = true;

				if (m_bChecked) SetState (BOX_LDOWN_1);
				else			SetState (BOX_LDOWN_2);
			}
		}
		break;  // WM_MOUSEMOVE


	case WM_LBUTTONUP:	
		mouseDown = false;
					
		if (PtInRect (&rc, pt)) {

			m_bChecked = (m_bChecked == true ? false : true);

			if (!spaceDown) {
				if (m_bChecked) SetState (BOX_ON);
				else			SetState (BOX_OFF);
			}
			else {
				if (m_bChecked) SetState (BOX_LDOWN_1);
				else			SetState (BOX_LDOWN_2);				
			}
		}
		else if (spaceDown)  // mouse released outside of control, but "space" still pressed
		{
			if (m_bChecked) SetState (BOX_LDOWN_1);
			else			SetState (BOX_LDOWN_2);
		}
		break;  // WM_LBUTTONUP
				// now call the default WNDPROC => it calls "ReleaseCapture"


	case WM_SETFOCUS:
		hasFocus = drawFocus;
		DrawFocus ();
		break;

	case WM_KILLFOCUS:
		if (spaceDown) {			// mouse pressed in another window, 
			spaceDown = false;		// while spaceDown
			if (m_bChecked) SetState (BOX_ON);
			else			SetState (BOX_OFF);
		}
		hasFocus = false;
		DrawFocus ();
		break;
	}
	
	return CallWindowProc (oProc, hWndBox,uMsg,wParam,lParam);
}


//-----------------------------------------------------------------------------
// Name:		DrawFocus
// Description:	Draws a focus rectangle
//
// Entry:		
//				bool forceDraw - this parameter can be one of the following:
//					 false : draw focus rect only if "hasFocus" has changed
//					 true  : draw focus rect regardles of previous value 
//							 of "hasFocus" 
//
//					hasFocus - has the following meaning:
//					true  : check box has the keyboard focus at the moment
//					false : check box is not focused at the moment
//-----------------------------------------------------------------------------
void CCheckBox::DrawFocus (bool forceDraw)
{	
	if (!drawFocus)   // draw focus rectangle?
		return;

	static bool prev = false;
	if (prev != hasFocus || forceDraw) {
		
		prev = hasFocus;

		HDC hdc = GetDC (m_hBox);
		DrawFocusRect (hdc,&rcFocus);
		ReleaseDC(m_hBox,hdc);
	}
}


//-----------------------------------------------------------------------------
// Name:		SetState
// Description:	Sets the new state and then blits the CB to the screen
//
// Entry:		
//				int nState - specifies the state of the CB and can be:
//					BOX_ON			== checked state
//					BOX_OFF			== unchecked state
//					BOX_LDOWN_1		== mouse button down in   checked state
//					BOX_LDOWN_2		== mouse button down in unchecked state
//					BOX_DISABLED_1	== CB disabled in checked state
//					BOX_DISABLED_2	== CB disabled in unchecked state
//-----------------------------------------------------------------------------
void CCheckBox::SetState (int state)
{
	if (state != m_nState) {
		m_nState = state;	

		HDC hdc = GetDC (m_hBox);
		BlitToScreen(hdc,state);
		ReleaseDC(m_hBox,hdc);

		if (hasFocus)
			DrawFocus (true);	
	}
}


//-----------------------------------------------------------------------------
// Name:		BlitToScreen
// Description:	Blits "m_bitmaps [nState]" to the screen
//
// Entry:		
//				HDC hdc - DC to blit to
//
//				int nState - specifies the state of the CB and can be:
//					BOX_ON
//					BOX_OFF
//					BOX_LDOWN_1
//					BOX_LDOWN_2
//					BOX_DISABLED_1
//					BOX_DISABLED_2
//-----------------------------------------------------------------------------
void CCheckBox::BlitToScreen (HDC hdc, int nState)
{
	HDC     destDC = ::CreateCompatibleDC (hdc);
	HBITMAP oDest;
	oDest = (HBITMAP) ::SelectObject (destDC,m_bitmaps [nState]);

	RECT rect;
	GetClientRect(m_hBox, &rect);

	int width  = rect.right  - rect.left;
	int height = rect.bottom - rect.top;

	if(height >= mHeight) {
		rect.top    += (height  - mHeight)/2;
		rect.bottom -= (height - mHeight)/2;
		height = rect.bottom - rect.top;
	}

	// Blit result to screen
	//	
	BitBlt(hdc, rect.left, rect.top, width, height, destDC, 0, 0, SRCCOPY);
	

	// Clean up
	//	
	SelectObject(destDC, oDest);
	
	DeleteDC(destDC);	
}


//-----------------------------------------------------------------------------
// Name:		GetMinHeight
// Description:	Calculates the minimal height of the CB based on m_font
//
// Entry:		
//				HDC hdc - DC into which m_font is selected
//			
//-----------------------------------------------------------------------------
int CCheckBox::GetMinHeight (HDC hdc)
{
	HFONT oFont = (HFONT) SelectObject (hdc,m_font);
	SIZE sz; GetTextExtentPoint32 (hdc,"Dummy",5, &sz);
	SelectObject (hdc,oFont);

	return sz.cy+1;
}


//-----------------------------------------------------------------------------
// Name:		PrepareBitmaps & PrepareState
// Description:	This methods prepare "m_bitmaps[???]" for all posible states
//
// Entry:		
//				HDC     hdc - DC used to create memory DCs for bitmap preparation
//
//				HBITMAP &bDest - this parameter is one of the following:
//						m_bitmaps [0] : BOX_ON  bitmap
//						m_bitmaps [1] : BOX_OFF bitmap
//						...
//						m_bitmaps [5] : BOX_DISABLED_2 bitmap
//
//				int nState - specifies the state of the CB and can be:
//					BOX_ON
//					BOX_OFF
//					BOX_LDOWN_1
//					BOX_LDOWN_2
//					BOX_DISABLED_1
//					BOX_DISABLED_2
//-----------------------------------------------------------------------------
void CCheckBox::PrepareBitmaps (HDC hdc)
{
	mHeight = GetMinHeight (hdc);
	if (mHeight < BOX_SIZE)
		mHeight = BOX_SIZE;
	

	PrepareState (hdc, m_bitmaps [0], BOX_ON);
	PrepareState (hdc, m_bitmaps [1], BOX_OFF);
	PrepareState (hdc, m_bitmaps [2], BOX_LDOWN_1);
	PrepareState (hdc, m_bitmaps [3], BOX_LDOWN_2);
	PrepareState (hdc, m_bitmaps [4], BOX_DISABLED_1);
	PrepareState (hdc, m_bitmaps [5], BOX_DISABLED_2);
}

void CCheckBox::PrepareState (HDC hdc, HBITMAP & bDest, int nState)
{
	RECT rect;
	GetClientRect(m_hBox, &rect);

	int width  = rect.right  - rect.left;
	int height = rect.bottom - rect.top;

	if(height >= mHeight) {
		rect.top    += (height - mHeight)/2;
		rect.bottom -= (height - mHeight)/2;
		height = rect.bottom - rect.top;
	}
	int o = (mHeight - BOX_SIZE) / 2;

	//
	// Create memory DCs and bitmaps to prepare the image
	//
	HDC srcDC  = ::CreateCompatibleDC (hdc);
	HDC maskDC = ::CreateCompatibleDC (hdc);
	HDC destDC = ::CreateCompatibleDC (hdc);


	HBITMAP bMask = ::CreateBitmap (BOX_SIZE,BOX_SIZE,1,1,NULL);
			bDest = ::CreateCompatibleBitmap (hdc,width,mHeight);		  

	HBITMAP oSrc  = (HBITMAP) ::SelectObject (srcDC, m_bSrc);	
	HBITMAP oMask = (HBITMAP) ::SelectObject (maskDC,bMask);
	HBITMAP oDest = (HBITMAP) ::SelectObject (destDC,bDest);	
	

	// Create mask - mask color: RGB(255,0,255)
	//
	COLORREF color = SetBkColor(srcDC, RGB(255,0,255));	


	RECT rc;				// rect = drawing rectangle inside client area of m_hBox
	rc.left	  = rc.top = 0;	// rc   = drawing rectangle inside destDC 
	rc.right  = width;		// right of rc = (width) of rect
	rc.bottom = height;     // bottom of rc = (height) of rect
	

	// copy the display surface where the CheckBox will be to destDC
	//
#if TRANSPARENT_CB
	BitBlt (destDC,0,0,width,height,hdc, rect.left, rect.top, SRCCOPY);
#else
	FillRect (destDC, &rc, m_brush);
#endif

	int l = 0;
	if (m_bLeftText) l = width - BOX_SIZE;		

	int x = BOX_SIZE*nState;  
	BitBlt(maskDC, 0, 0, BOX_SIZE, BOX_SIZE, srcDC, x, 0, SRCCOPY);
	BitBlt(destDC, l, o, BOX_SIZE, BOX_SIZE, srcDC, x, 0, SRCINVERT); 
	BitBlt(destDC, l, o, BOX_SIZE, BOX_SIZE,maskDC, 0, 0, SRCAND); 
	BitBlt(destDC, l, o, BOX_SIZE, BOX_SIZE, srcDC, x, 0, SRCINVERT);


	// Draw control text
	//
	DrawText_ (destDC,rc,nState);
	rcFocus.top    += rect.top;
	rcFocus.bottom += rect.top;
	rcFocus.left  -= 1;
	rcFocus.right += 1;
	
	// Clean up
	//
	SelectObject(srcDC,  oSrc);
	SelectObject(maskDC, oMask);
	SelectObject(destDC, oDest);
	
	DeleteDC(srcDC);
	DeleteDC(maskDC);
	DeleteDC(destDC);

	DeleteObject(bMask);
}


//-----------------------------------------------------------------------------
// Name:		DrawText_
// Description:	Draws the text for the CB to destDC
//				(into which one of the "m_bitmaps [???]" is selected)
//
// Entry:		
//				HDC destDC - DC to which the text is drawed
//							 ("m_bitmaps [???]" is selected into it)
//
//				RECT rc    - part of client area (of the CB) to which
//							 text is output
//			
//-----------------------------------------------------------------------------
void CCheckBox::DrawText_ (HDC destDC, RECT rc, int nState)
{	
	int oState = SaveDC (destDC);

	SetBkMode (destDC, TRANSPARENT);
	SelectObject (destDC,m_font);


	unsigned style = 0;
	if (m_bLeftText) {
		rc.right -= BOX_SIZE+6;  style = DT_RIGHT;
	}
	else {
		rc.left  += BOX_SIZE+5;  style = DT_LEFT;
	}
	rc.bottom -= 1;				// rc   = drawing rectangle inside destDC  
	

	if (nState == BOX_DISABLED_1 || nState == BOX_DISABLED_2) 
	{
		SetTextColor (destDC, GetSysColor (COLOR_3DHILIGHT));
		rc.left  += 1;	rc.top   += 1;
		rc.right += 1;	rc.bottom+= 1;
		DrawText (destDC, m_text, -1, &rc, DT_SINGLELINE | DT_VCENTER | style);
			
		SetTextColor (destDC, GetSysColor (COLOR_3DSHADOW));
		rc.left  -= 1;	rc.top   -= 1;
		rc.right -= 1;	rc.bottom-= 1;
		DrawText (destDC, m_text, -1, &rc, DT_SINGLELINE | DT_VCENTER | style);
	}
	else
		DrawText (destDC, m_text, -1, &rc, DT_SINGLELINE | DT_VCENTER | style);


	// calculate rcFocus
	//
	int r_t = rc.right;				// right side of text rectangle
	
	DrawText (destDC, m_text, -1, &rc, DT_SINGLELINE | DT_VCENTER | style | DT_CALCRECT);

	if (m_bLeftText) {
		int d = rc.right - rc.left; // text width
		d = r_t - d;
		rc.left  += d; 
		rc.right += d;
	}
	rcFocus = rc;

	RestoreDC (destDC, oState);
}
