//
//	File:		CCheckBox.h
//
//	Class:		CCheckBox (API)
//
//
//	Version:	1.0
//
//	Created:	December 2002, January 2003
//	Updated:	11/January/2003
//
//	Compiler:	Visual C++ 6.0
//	Tested on:	Win2000, SP2
//
//
//	Author:		Robert Kuster		robert.kuster@email.si
//	Copyright   (C) 2003
//
// 
//	Disclaimer
//	----------
//	This source code (CCheckBox.h and CCheckBox.cpp) is provided "as is" with NO 
//	expressed or implied warranty. The user must assume the entire risk of using it.
//	No responsibilities for possible damages or even functionality can be taken.
//
//
//	Terms of use
//	------------
//	This source code is free for personal use and freeware applications.  
//	For commercial use a simple email would be nice.
//
/////////////////////////////////////////////////////////////////////////////


#if !defined _CHECK_BOX_WINAPI_RK_
#define _CHECK_BOX_WINAPI_RK_


/////////////////////////////////////////////////////////////////////////////
// CCheckBox (CB) class
//
class CCheckBox {

public:		
	CCheckBox	  (int rcID,int nDimension);	// rcID		  : bitmap resource ID;		
												// nDimension : bitmap dimension;
	void Subclass (UINT nID, HWND hParent);		// nID		  : ID of CB (to subclass);
	void Subclass (HWND hCB, HWND hParent);		// hCB		  : handle to CB (to subclass);
												// hParent	  : check box parent;
	virtual ~CCheckBox ();

	void SetCheck (int nCheck);					// nCheck == 1 => checked, 0 => unchecked
	int  GetCheck () { return m_bChecked; }		// returns 1 if checked, 0 if unchecked

	static HINSTANCE m_hInst; // handle to application instance
	static bool drawFocus;	  // draw focus rectangle?


protected:
	enum {
		BOX_ON = 0,			// Column 0 : checked state
		BOX_OFF = 1,		// Column 1 : unchecked state
		BOX_LDOWN_1	 = 2,	// Column 2 : mouse down in checked state
		BOX_LDOWN_2	 = 3,	// Column 3 : mouse down in unchecked state
		BOX_DISABLED_1 = 4,	// Column 4 : CB disabled in checked state
		BOX_DISABLED_2 = 5	// Column 5 : CB disabled in unchecked state
	};	

	HWND m_hDlg;			// handle to parent dialog
	HWND m_hBox;			// handle to check box control

	bool m_bLeftText;		// BS_LEFTTEXT || BS_RIGHTBUTTON || WS_EX_RIGHT ??
	bool m_bDisabled;
	bool m_bChecked;
	int  m_nState;			// state of check box: BOX_ON, BOX_OFF,...

	int  BOX_SIZE;		    // size of bitmaps (BOX_SIZE X BOX_SIZE)
	int  m_rcID;			// resource ID (m_bSrc)

	DWORD  m_clr;			// background color == COLOR_3DFACE
	HBRUSH m_brush;			// background brush

	HBITMAP m_bSrc;			// source bitmap
	HBITMAP m_bitmaps [6];  // bitmaps for all states BOX_ON, BOX_OFF,...BOX_DISABLED_2
	
	bool inRect;			// mouse cursor over CB?
	bool spaceDown;			// space button pressed?
	bool mouseDown;			// mouse button pressed?


	HFONT m_font;			// control font
	char  m_text [64];		// control text
	void  DrawText_ (HDC, RECT, int);
	
	int mHeight;
	int GetMinHeight (HDC);	// calculates the minimal height 'mHeight'
							// based on m_font

	bool prepare_bitmaps;
	void PrepareBitmaps (HDC);              // prepares bitmpas for all states
	void PrepareState (HDC,HBITMAP &,int);  // create CB bitmap for specified state

	void SetState     (int nState);         // set state (BOX_ON,BOX_OFF,...BOX_DISABLED_2)
	void BlitToScreen (HDC hdc, int nState);// blit m_bitmaps[nState] to screen

	bool hasFocus;							// has focus?
	RECT rcFocus;							// focus rectangle
	void DrawFocus (bool forceDraw=false);

	void Subclass_X ();						// sublcasses the CB
	
	static LRESULT CALLBACK StaticProc (HWND,UINT,WPARAM,LPARAM);
	LRESULT WndProc (HWND,UINT,WPARAM,LPARAM);  // new WNDPROC for subclassed CB control
	WNDPROC oProc;								// old WNDPROC
};

#endif // !defined(_CHECK_BOX_MFC_RK_)